{% extends "appbuilder/baselayout.html" %}

  {% block head_css %}
    {{super()}}
    <link rel="stylesheet" type="text/css" href="/static/assets/stylesheets/panoramix.css" />
  {% endblock %}

  {% block head_js %}
    {{super()}}
    <script src="/static/assets/javascripts/dist/css-theme.entry.js"></script>
  {% endblock %}
