{% extends "panoramix/basic.html" %}
{% block body %}
<div class="container">
  <div class="header">
    <h3><i class='fa fa-star'></i> Featured Datasets </h3>
  </div>
  <hr/>
  <table class="table table-hover dataTable table-bordered" id="dataset-table" style="display:None">
      <thead>
          <tr>
              <th>Table</th>
              <th>Database</th>
              <th>Owner</th>
              <th></th>
          </tr>
      </thead>
      <tbody>
      {% for dataset in featured_datasets %}
          <tr>
              <td>
                <div class="intable-longtext">
                  <h4>{{ dataset.table_name }}</h4>
                  <p>{{ utils.markdown(dataset.description) | safe }}</p>
                </div>
              </td>
              <td class="small_table">{{ dataset.database }}</td>
              <td class="small_table">{{ dataset.owner or '' }}</td>
              <td class="small_table"><a class="btn btn-default" href="{{ dataset.default_endpoint }}"><i class='fa fa-line-chart'/></a></td>
         </tr>
      {% endfor %}
      </tbody>
  </table>
  <hr/>
</div>
{% endblock %}

{% block tail_js %}
  {{ super() }}
  <script src="/static/assets/javascripts/dist/featured.entry.js"></script>
{% endblock %}
