<html>
  <head>
    <title>
      {% block title %}
        {% if appbuilder and appbuilder.app_name %} {{ appbuilder.app_name }} {% endif %}
      {% endblock %}
    </title>
    {% block head_meta %}{% endblock %}
    {% block head_css %}
      <link rel="stylesheet" type="text/css" href="/static/assets/node_modules/font-awesome/css/font-awesome.min.css" />
    {% endblock %}
    {% block head_js %}
      <script src="/static/assets/javascripts/dist/css-theme.entry.js"></script>
    {% endblock %}
  </head>

  <body>
    {% block navbar %}
      {% if not viz or viz.request.args.get("standalone") != "true" %}
        <header class="top" role="header">
          {% include 'appbuilder/navbar.html' %}
        </header>
      {% endif %}
    {% endblock %}

    {% block body %}
      <div id="app">
        Oops! React.js is not working properly.
      </div>
    {% endblock %}

    {% block tail_js %}
    {% endblock %}
  </body>
</html>
