.gridster .widget.pivot_table {
    overflow: auto !important;
}

.table tr>th {
    padding: 1px 5px !important;
    font-size: small !important;
}

.table tr>td {
    padding: 1px 5px !important;
    font-size: small !important;
}
