body {
    margin: 0px !important;
}

input.form-control {
    background-color: white;
}

.col-left-fixed {
    width:350px;
    position: absolute;
    float: left;
}
.col-offset {
    margin-left: 365px;
}

.modal-content.css {
    transition: opacity 0.5s ease;
    opacity: 0.5;
    border-color: black;
}

.modal-content.css:hover {
    transition: opacity 0.5s ease;
    opacity: 1;
}

.slice_description{
    padding: 8px;
    margin: 5px;
    border: 1px solid #DDD;
    background-color: #F8F8F8;
    border-radius: 5px;
    font-size: 12px;
}

.slice_info{
    cursor: pointer;
}

.padded{
    padding: 10px;
}

.intable-longtext{
    max-height: 200px;
    overflow: auto;
}

.slice_container {
    //height: 100%;
}
.container-fluid {
    text-align: left;
}
input[type="checkbox"] {
  display: inline-block;
  width: 16px;
  height: 16px;
  float: right;
}
form div {
  padding-top: 1px;
}
.navbar-brand a {
  color: white;
}

.header span{
    margin-left: 3px;
}

#timer {
    width: 80px;
    text-align: right;
}

.notbtn {
    cursor: default;
}
hr {
    margin-top: 15px;
    margin-bottom: 15px;
}

span.title-block {
    background-color: #EEE;
    border-radius: 4px;
    padding: 6px 12px;
    margin: 0px 10px;
    font-size: 20px;
}

fieldset.fs-style {
    font-family: Verdana, Arial, sans-serif;
    font-size: small;
    font-weight: normal;
    border: 1px solid #CCC;
    background-color: #F4F4F4;
    border-radius: 6px;
    padding: 10px;
    margin: 0px 0px 10px 0px;
}
legend.legend-style {
    font-size: 14px;
    padding: 0px 6px;
    cursor: pointer;
    margin: 0px;
    color: #444;
    background-color: transparent;
    font-weight: bold;
}
.nvtooltip {
    //position: relative !important;
    z-index: 888;
}
.nvtooltip table td{
    font-size: 11px !important;
}
legend {
    width: auto;
    border-bottom: 0px;
}
.navbar {
  -webkit-box-shadow: 0px 3px 3px #AAA;
  -moz-box-shadow: 0px 3px 3px #AAA;
  box-shadow: 0px 3px 3px #AAA;
  z-index: 999;
}
.panel.panel-primary {
  margin: 10px;
}

.index .carousel img {
  max-height: 500px;
}
.index .carousel {
  overflow: hidden;
  height: 500px;
}
.index .carousel-caption h1 {
  font-size: 80px;
}
.index .carousel-caption p {
  font-size: 20px;
}
.index div.carousel-caption{
  background: rgba(0,0,0,0.5);
  border-radius: 20px;
  top: 150px;
  bottom: auto !important;
}
.index .carousel-inner > .item > img {
  margin: 0 auto;
}
.index {
  margin: -20px;
}
.index .carousel-indicators li {
  background-color: #AAA;
  border: 1px solid black;
}

.index .carousel-indicators .active {
  background-color: #000;
  border: 5px solid black;
}

.datasource form div.form-control {
  margin-bottom: 5px !important;
}
.datasource form input.form-control {
  margin-bottom: 5px !important;
}
.datasource .tooltip-inner {
  max-width: 350px;
}
img.loading {
  width: 40px;
}

.dashboard a i {
  cursor: pointer;
}
.dashboard i.drag {
  cursor: move !important;
}
.dashboard .gridster .preview-holder {
  z-index: 1;
  position: absolute;
  background-color: #AAA;
  border-color: #AAA;
  opacity: 0.3;
}
.gridster li.widget{
  list-style-type: none;
  border-radius: 0;
  padding: 10px;
  margin: 5px;
  border: 1px solid #ccc;
  box-shadow: 2px 1px 5px -2px #aaa;
  overflow: hidden;
  background-color: #fff;
}
.dashboard .gridster .dragging,
.dashboard .gridster .resizing {
  opacity: 0.5;
}
.dashboard img.loading {
  width: 20px;
  margin: 5px;
}
.dashboard .title {
  text-align: center;
}
.dashboard .slice_title {
  text-align: center;
  font-weight: bold;
  font-size: 14px;
  padding: 5px;
}
.dashboard div.slice_content {
  width: 100%;
  height: 100%;
}
.dashboard table.slice_header {
  width: 100%;
  height: 20px;
}
.dashboard  div.nvtooltip {
  z-index: 888;  /* this lets tool tips go on top of other slices  */
}
.dashboard td.icons {
  width: 50px;
}
.dashboard td.icons nobr {
  display: none;
}
div.header {
  font-weight: bold;
}
li.widget:hover {
  z-index: 1000;
}
