.sunburst text.middle{
    text-anchor: middle;
}

.sunburst #sequence {
}

.sunburst #legend {
  padding: 10px 0 0 3px;
}

.sunburst #sequence text, #legend text {
  font-weight: 600;
  fill: #fff;
}

.sunburst path {
  stroke: #fff;
}

.sunburst #percentage {
  font-size: 2.5em;
}

