<html>
  <head>
    <script src="/static/assets/javascripts/dist/standalone.entry.js"></script>
    <link rel="stylesheet" type="text/css" href="/static/assets/stylesheets/panoramix.css" />
    {% set CSS_THEME = appbuilder.get_app.config.get("CSS_THEME") %}
    {% set height = request.args.get("height", 700) %}
    {% if CSS_THEME %}
      <link rel="stylesheet" type="text/css" href="{{ CSS_THEME }}" />
    {% endif %}
  </head>
  <body>
    <div
      id="{{ viz.token }}"
      class="widget viz slice {{ viz.viz_type }}"
      data-slice="{{ viz.json_data }}"
      style="height: {{ height }}px;">
      <img src="{{ url_for("static", filename="img/loading.gif") }}" class="loading" alt="loading">
      <div id="{{ viz.token }}_con" class="slice_container" style="height: 100%; width: 100%"></div>
    </div>
  </body>
</html>
