.sankey .node rect {
  cursor: move;
  fill-opacity: .9;
  shape-rendering: crispEdges;
}

.sankey .node text {
  pointer-events: none;
  text-shadow: 0 1px 0 #fff;
}

.sankey .link {
  fill: none;
  stroke: #000;
  stroke-opacity: .2;
}

.sankey .link:hover {
  stroke-opacity: .5;
}
