


{% set menu = appbuilder.menu %}
{% set languages = appbuilder.languages %}

<div class="navbar navbar-static-top {{menu.extra_classes}}" role="navigation">
   <div class="container-fluid">
        <div class="navbar-header">
            <button type="button" class="navbar-toggle" data-toggle="collapse" data-target=".navbar-collapse">
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
            </button>
            {% if appbuilder.app_icon %}
                <a class="navbar-brand" style="padding:10px;" href="{{appbuilder.get_url_for_index}}">
                  <img width="30" src="{{appbuilder.app_icon}}">
                </a>
            {% endif %}
                <span class="navbar-brand">
                <a href="{{appbuilder.get_url_for_index}}">
                  {{ appbuilder.app_name }}
                </a>
                </span>
        </div>
        <div class="navbar-collapse collapse">
            <ul class="nav navbar-nav">
                {% include 'appbuilder/navbar_menu.html' %}
            </ul>
            <ul class="nav navbar-nav navbar-right">
                {% include 'appbuilder/navbar_right.html' %}
            </ul>
        </div>
   </div>
</div>
