{% if viz.form_data.get("json") == "true" %}
  {{ viz.get_json() }}
{% else %}

  {% if viz.request.args.get("standalone") == "true" %}
    {% extends 'panoramix/standalone.html' %}
  {% else %}
    {% extends 'panoramix/explore.html' %}
  {% endif %}

{% endif %}
